<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <title>SPIRES: Results</title>
  <style>
    body {
      font-family: Arial, sans-serif;
      margin: 0;
      padding: 0;
    }
    
    .container {
      max-width: 800px;
      margin: 0 auto;
      padding: 20px;
    }
    
    .container a {
      color: #007bff;
      text-decoration: none;
    }
  </style>
</head>
<body>
<div class="container">
  <a href="/">Try another!</a>
</div>

{{ inner_html|safe }}

</body>
</html>
